@@ -7,11 +7,12 @@
 #include <jxl/color_encoding.h>
 #include <jxl/encode.h>
 #include <jxl/types.h>
-#include <stddef.h>
 
 #include <algorithm>
+#include <cstddef>
 #include <cstdint>
 #include <cstdio>
+#include <cstdlib>
 #include <cstring>
 #include <memory>
 #include <sstream>
@@ -22,10 +23,11 @@
 #include "lib/extras/common.h"
 #include "lib/extras/dec/color_hints.h"
 #include "lib/extras/dec/decode.h"
-#include "lib/extras/dec/pnm.h"
 #include "lib/extras/enc/encode.h"
 #include "lib/extras/packed_image.h"
 #include "lib/jxl/base/byte_order.h"
+#include "lib/jxl/base/compiler_specific.h"
+#include "lib/jxl/base/data_parallel.h"
 #include "lib/jxl/base/random.h"
 #include "lib/jxl/base/span.h"
 #include "lib/jxl/base/status.h"
@@ -35,16 +37,18 @@
 
 namespace jxl {
 
-using test::ThreadPoolForTests;
+using ::jxl::test::ThreadPoolForTests;
 
 namespace extras {
+
+Status PnmParseSigned(Bytes str, double* v);
+Status PnmParseUnsigned(Bytes str, size_t* v);
+
 namespace {
 
-using ::testing::AllOf;
-using ::testing::Contains;
-using ::testing::Field;
-using ::testing::IsEmpty;
-using ::testing::SizeIs;
+Span<const uint8_t> MakeSpan(const char* str) {
+  return Bytes(reinterpret_cast<const uint8_t*>(str), strlen(str));
+}
 
 std::string ExtensionFromCodec(Codec codec, const bool is_gray,
                                const bool has_alpha,
@@ -112,15 +116,15 @@ JxlColorEncoding CreateTestColorEncoding(bool is_gray) {
   // Roundtrip through internal color encoding to fill in primaries and white
   // point CIE xy coordinates.
   ColorEncoding c_internal;
-  JXL_CHECK(c_internal.FromExternal(c));
+  EXPECT_TRUE(c_internal.FromExternal(c));
   c = c_internal.ToExternal();
   return c;
 }
 
 std::vector<uint8_t> GenerateICC(JxlColorEncoding color_encoding) {
   ColorEncoding c;
-  JXL_CHECK(c.FromExternal(color_encoding));
-  JXL_CHECK(!c.ICC().empty());
+  EXPECT_TRUE(c.FromExternal(color_encoding));
+  EXPECT_TRUE(!c.ICC().empty());
   return c.ICC();
 }
 
@@ -233,13 +237,17 @@ void CreateTestImage(const TestImageParams& params, PackedPixelFile* ppf) {
   ppf->icc = GenerateICC(color_encoding);
   ppf->color_encoding = color_encoding;
 
-  PackedFrame frame(params.xsize, params.ysize, params.PixelFormat());
+  JXL_TEST_ASSIGN_OR_DIE(
+      PackedFrame frame,
+      PackedFrame::Create(params.xsize, params.ysize, params.PixelFormat()));
   FillPackedImage(params.bits_per_sample, &frame.color);
   if (params.add_extra_channels) {
     for (size_t i = 0; i < 7; ++i) {
       JxlPixelFormat ec_format = params.PixelFormat();
       ec_format.num_channels = 1;
-      PackedImage ec(params.xsize, params.ysize, ec_format);
+      JXL_TEST_ASSIGN_OR_DIE(
+          PackedImage ec,
+          PackedImage::Create(params.xsize, params.ysize, ec_format));
       FillPackedImage(params.bits_per_sample, &ec);
       frame.extra_channels.emplace_back(std::move(ec));
       PackedExtraChannel pec;
@@ -335,10 +343,10 @@ TEST(CodecTest, TestRoundTrip) {
             params.add_alpha = add_alpha;
             params.big_endian = big_endian;
             params.add_extra_channels = false;
-            TestRoundTrip(params, &pool);
+            TestRoundTrip(params, pool.get());
             if (codec == Codec::kPNM && add_alpha) {
               params.add_extra_channels = true;
-              TestRoundTrip(params, &pool);
+              TestRoundTrip(params, pool.get());
             }
           }
         }
@@ -371,7 +379,7 @@ TEST(CodecTest, LosslessPNMRoundtrip) {
       EncodedImage encoded;
       auto encoder = Encoder::FromExtension(extension);
       ASSERT_TRUE(encoder.get());
-      ASSERT_TRUE(encoder->Encode(ppf, &encoded, &pool));
+      ASSERT_TRUE(encoder->Encode(ppf, &encoded, pool.get()));
       ASSERT_EQ(encoded.bitstreams.size(), 1);
       ASSERT_EQ(orig.size(), encoded.bitstreams[0].size());
       EXPECT_EQ(0,
@@ -380,7 +388,40 @@ TEST(CodecTest, LosslessPNMRoundtrip) {
   }
 }
 
-TEST(CodecTest, TestPNM) { TestCodecPNM(); }
+TEST(CodecTest, TestPNM) {
+  size_t u = 77777;  // Initialized to wrong value.
+  double d = 77.77;
+// Failing to parse invalid strings results in a crash if `JXL_CRASH_ON_ERROR`
+// is defined and hence the tests fail. Therefore we only run these tests if
+// `JXL_CRASH_ON_ERROR` is not defined.
+#if (!JXL_CRASH_ON_ERROR)
+  ASSERT_FALSE(PnmParseUnsigned(MakeSpan(""), &u));
+  ASSERT_FALSE(PnmParseUnsigned(MakeSpan("+"), &u));
+  ASSERT_FALSE(PnmParseUnsigned(MakeSpan("-"), &u));
+  ASSERT_FALSE(PnmParseUnsigned(MakeSpan("A"), &u));
+
+  ASSERT_FALSE(PnmParseSigned(MakeSpan(""), &d));
+  ASSERT_FALSE(PnmParseSigned(MakeSpan("+"), &d));
+  ASSERT_FALSE(PnmParseSigned(MakeSpan("-"), &d));
+  ASSERT_FALSE(PnmParseSigned(MakeSpan("A"), &d));
+#endif
+  ASSERT_TRUE(PnmParseUnsigned(MakeSpan("1"), &u));
+  ASSERT_TRUE(u == 1);
+
+  ASSERT_TRUE(PnmParseUnsigned(MakeSpan("32"), &u));
+  ASSERT_TRUE(u == 32);
+
+  ASSERT_TRUE(PnmParseSigned(MakeSpan("1"), &d));
+  ASSERT_TRUE(d == 1.0);
+  ASSERT_TRUE(PnmParseSigned(MakeSpan("+2"), &d));
+  ASSERT_TRUE(d == 2.0);
+  ASSERT_TRUE(PnmParseSigned(MakeSpan("-3"), &d));
+  ASSERT_TRUE(std::abs(d - -3.0) < 1E-15);
+  ASSERT_TRUE(PnmParseSigned(MakeSpan("3.141592"), &d));
+  ASSERT_TRUE(std::abs(d - 3.141592) < 1E-15);
+  ASSERT_TRUE(PnmParseSigned(MakeSpan("-3.141592"), &d));
+  ASSERT_TRUE(std::abs(d - -3.141592) < 1E-15);
+}
 
 TEST(CodecTest, FormatNegotiation) {
   const std::vector<JxlPixelFormat> accepted_formats = {
@@ -432,15 +473,17 @@ TEST(CodecTest, EncodeToPNG) {
   ASSERT_TRUE(extras::DecodeBytes(Bytes(original_png), ColorHints(), &ppf));
 
   const JxlPixelFormat& format = ppf.frames.front().color.format;
-  ASSERT_THAT(
-      png_encoder->AcceptedFormats(),
-      Contains(AllOf(Field(&JxlPixelFormat::num_channels, format.num_channels),
-                     Field(&JxlPixelFormat::data_type, format.data_type),
-                     Field(&JxlPixelFormat::endianness, format.endianness))));
+  const auto& format_matcher = [&format](const JxlPixelFormat& candidate) {
+    return (candidate.num_channels == format.num_channels) &&
+           (candidate.data_type == format.data_type) &&
+           (candidate.endianness == format.endianness);
+  };
+  const auto formats = png_encoder->AcceptedFormats();
+  ASSERT_TRUE(std::any_of(formats.begin(), formats.end(), format_matcher));
   EncodedImage encoded_png;
   ASSERT_TRUE(png_encoder->Encode(ppf, &encoded_png, pool));
-  EXPECT_THAT(encoded_png.icc, IsEmpty());
-  ASSERT_THAT(encoded_png.bitstreams, SizeIs(1));
+  EXPECT_TRUE(encoded_png.icc.empty());
+  ASSERT_EQ(encoded_png.bitstreams.size(), 1);
 
   PackedPixelFile decoded_ppf;
   ASSERT_TRUE(extras::DecodeBytes(Bytes(encoded_png.bitstreams.front()),
