@@ -5,6 +5,8 @@
 
 #include "lib/jxl/jpeg/jpeg_data.h"
 
+#include <jxl/types.h>
+
 #include "lib/jxl/base/printf_macros.h"
 #include "lib/jxl/base/status.h"
 #include "lib/jxl/common.h"  // kMaxNumPasses, JPEGXL_ENABLE_TRANSCODE_JPEG
@@ -76,12 +78,12 @@ Status JPEGData::VisitFields(Visitor* visitor) {
     if (marker_order.size() > 16384) {
       return JXL_FAILURE("Too many markers: %" PRIuS "\n", marker_order.size());
     }
-    for (size_t i = 0; i < marker_order.size(); i++) {
-      JXL_RETURN_IF_ERROR(VisitMarker(&marker_order[i], visitor, &info));
+    for (uint8_t& marker : marker_order) {
+      JXL_RETURN_IF_ERROR(VisitMarker(&marker, visitor, &info));
     }
     if (!marker_order.empty()) {
       // Last marker should always be EOI marker.
-      JXL_CHECK(marker_order.back() == 0xd9);
+      JXL_ENSURE(marker_order.back() == 0xd9);
     }
   }
 
@@ -92,10 +94,10 @@ Status JPEGData::VisitFields(Visitor* visitor) {
     com_data.resize(info.num_com_markers);
     scan_info.resize(info.num_scans);
   }
-  JXL_ASSERT(app_data.size() == info.num_app_markers);
-  JXL_ASSERT(app_marker_type.size() == info.num_app_markers);
-  JXL_ASSERT(com_data.size() == info.num_com_markers);
-  JXL_ASSERT(scan_info.size() == info.num_scans);
+  JXL_ENSURE(app_data.size() == info.num_app_markers);
+  JXL_ENSURE(app_marker_type.size() == info.num_app_markers);
+  JXL_ENSURE(com_data.size() == info.num_com_markers);
+  JXL_ENSURE(scan_info.size() == info.num_scans);
   for (size_t i = 0; i < app_data.size(); i++) {
     auto& app = app_data[i];
     // Encodes up to 8 different values.
@@ -173,8 +175,8 @@ Status JPEGData::VisitFields(Visitor* visitor) {
     components.resize(num_components);
   }
   if (component_type == JPEGComponentType::kCustom) {
-    for (size_t i = 0; i < components.size(); i++) {
-      JXL_RETURN_IF_ERROR(visitor->Bits(8, 0, &components[i].id));
+    for (auto& component : components) {
+      JXL_RETURN_IF_ERROR(visitor->Bits(8, 0, &component.id));
     }
   } else if (component_type == JPEGComponentType::kGray) {
     components[0].id = 1;
@@ -214,7 +216,7 @@ Status JPEGData::VisitFields(Visitor* visitor) {
     huffman_code.resize(num_huff);
   }
   for (JPEGHuffmanCode& hc : huffman_code) {
-    bool is_ac = hc.slot_id >> 4;
+    bool is_ac = ((hc.slot_id >> 4) != 0);
     uint32_t id = hc.slot_id & 0xF;
     JXL_RETURN_IF_ERROR(visitor->Bool(false, &is_ac));
     JXL_RETURN_IF_ERROR(visitor->Bits(2, 0, &id));
@@ -247,7 +249,7 @@ Status JPEGData::VisitFields(Visitor* visitor) {
       return JXL_FAILURE("Missing EOI symbol");
     }
     // Last element, denoting EOI, have to be 1 after the loop.
-    JXL_ASSERT(value_slots[4] == 1);
+    JXL_ENSURE(value_slots[4] == 1);
     size_t num_values = 1;
     for (size_t i = 0; i < 4; ++i) num_values += hwy::PopCount(value_slots[i]);
     if (num_values != num_symbols) {
@@ -320,11 +322,11 @@ Status JPEGData::VisitFields(Visitor* visitor) {
       scan.extra_zero_runs.resize(num_extra_zero_runs);
     }
     last_block_idx = -1;
-    for (size_t i = 0; i < scan.extra_zero_runs.size(); ++i) {
-      uint32_t& block_idx = scan.extra_zero_runs[i].block_idx;
-      JXL_RETURN_IF_ERROR(visitor->U32(
-          Val(1), BitsOffset(2, 2), BitsOffset(4, 5), BitsOffset(8, 20), 1,
-          &scan.extra_zero_runs[i].num_extra_zero_runs));
+    for (auto& extra_zero_run : scan.extra_zero_runs) {
+      uint32_t& block_idx = extra_zero_run.block_idx;
+      JXL_RETURN_IF_ERROR(visitor->U32(Val(1), BitsOffset(2, 2),
+                                       BitsOffset(4, 5), BitsOffset(8, 20), 1,
+                                       &extra_zero_run.num_extra_zero_runs));
       block_idx -= last_block_idx + 1;
       JXL_RETURN_IF_ERROR(visitor->U32(Val(0), BitsOffset(3, 1),
                                        BitsOffset(5, 9), BitsOffset(28, 41), 0,
@@ -362,13 +364,13 @@ Status JPEGData::VisitFields(Visitor* visitor) {
       for (uint32_t i = 0; i < nbit; i++) {
         bool bbit = false;
         JXL_RETURN_IF_ERROR(visitor->Bool(false, &bbit));
-        padding_bits.push_back(bbit);
+        padding_bits.push_back(TO_JXL_BOOL(bbit));
       }
     } else {
       for (uint8_t& bit : padding_bits) {
-        bool bbit = bit;
+        bool bbit = FROM_JXL_BOOL(bit);
         JXL_RETURN_IF_ERROR(visitor->Bool(false, &bbit));
-        bit = bbit;
+        bit = TO_JXL_BOOL(bbit);
       }
     }
   }
@@ -416,7 +418,7 @@ Status JPEGData::VisitFields(Visitor* visitor) {
   // Apply postponed actions.
   if (visitor->IsReading()) {
     tail_data.resize(tail_data_len);
-    JXL_ASSERT(inter_marker_data_sizes.size() == info.num_intermarker);
+    JXL_ENSURE(inter_marker_data_sizes.size() == info.num_intermarker);
     inter_marker_data.reserve(info.num_intermarker);
     for (size_t i = 0; i < info.num_intermarker; ++i) {
       inter_marker_data.emplace_back(inter_marker_data_sizes[i]);
