@@ -5,13 +5,14 @@
 
 #include "lib/jxl/render_pipeline/stage_noise.h"
 
+#include "lib/jxl/base/sanitizers.h"
+#include "lib/jxl/noise.h"
+
 #undef HWY_TARGET_INCLUDE
 #define HWY_TARGET_INCLUDE "lib/jxl/render_pipeline/stage_noise.cc"
 #include <hwy/foreach_target.h>
 #include <hwy/highway.h>
 
-#include "lib/jxl/sanitizers.h"
-
 HWY_BEFORE_NAMESPACE();
 namespace jxl {
 namespace HWY_NAMESPACE {
@@ -61,9 +62,10 @@ class StrengthEvalLut {
 #endif
   {
 #if HWY_TARGET != HWY_SCALAR
-    uint32_t lut[8];
-    memcpy(lut, noise_params.lut, sizeof(lut));
-    for (size_t i = 0; i < 8; i++) {
+    uint32_t lut[NoiseParams::kNumNoisePoints];
+    memcpy(lut, noise_params.lut.data(),
+           NoiseParams::kNumNoisePoints * sizeof(uint32_t));
+    for (size_t i = 0; i < NoiseParams::kNumNoisePoints; i++) {
       low16_lut[2 * i] = (lut[i] >> 0) & 0xFF;
       low16_lut[2 * i + 1] = (lut[i] >> 8) & 0xFF;
       high16_lut[2 * i] = (lut[i] >> 16) & 0xFF;
@@ -154,11 +156,11 @@ void AddNoiseToRGB(const D d, const Vec<D> rnd_noise_r,
 class AddNoiseStage : public RenderPipelineStage {
  public:
   AddNoiseStage(const NoiseParams& noise_params,
-                const ColorCorrelationMap& cmap, size_t first_c)
+                const ColorCorrelation& color_correlation, size_t first_c)
       : RenderPipelineStage(RenderPipelineStage::Settings::Symmetric(
             /*shift=*/0, /*border=*/0)),
         noise_params_(noise_params),
-        cmap_(cmap),
+        color_correlation_(color_correlation),
         first_c_(first_c) {}
 
   Status ProcessRow(const RowInfo& input_rows, const RowInfo& output_rows,
@@ -174,8 +176,8 @@ class AddNoiseStage : public RenderPipelineStage {
     // normalizer is half of what it was before (0.5).
     const auto norm_const = Set(d, 0.22f);
 
-    float ytox = cmap_.YtoXRatio(0);
-    float ytob = cmap_.YtoBRatio(0);
+    float ytox = color_correlation_.YtoXRatio(0);
+    float ytob = color_correlation_.YtoBRatio(0);
 
     const size_t xsize_v = RoundUpTo(xsize, Lanes(d));
 
@@ -225,14 +227,15 @@ class AddNoiseStage : public RenderPipelineStage {
 
  private:
   const NoiseParams& noise_params_;
-  const ColorCorrelationMap& cmap_;
+  const ColorCorrelation& color_correlation_;
   size_t first_c_;
 };
 
 std::unique_ptr<RenderPipelineStage> GetAddNoiseStage(
-    const NoiseParams& noise_params, const ColorCorrelationMap& cmap,
+    const NoiseParams& noise_params, const ColorCorrelation& color_correlation,
     size_t noise_c_start) {
-  return jxl::make_unique<AddNoiseStage>(noise_params, cmap, noise_c_start);
+  return jxl::make_unique<AddNoiseStage>(noise_params, color_correlation,
+                                         noise_c_start);
 }
 
 class ConvolveNoiseStage : public RenderPipelineStage {
@@ -303,9 +306,9 @@ HWY_EXPORT(GetAddNoiseStage);
 HWY_EXPORT(GetConvolveNoiseStage);
 
 std::unique_ptr<RenderPipelineStage> GetAddNoiseStage(
-    const NoiseParams& noise_params, const ColorCorrelationMap& cmap,
+    const NoiseParams& noise_params, const ColorCorrelation& color_correlation,
     size_t noise_c_start) {
-  return HWY_DYNAMIC_DISPATCH(GetAddNoiseStage)(noise_params, cmap,
+  return HWY_DYNAMIC_DISPATCH(GetAddNoiseStage)(noise_params, color_correlation,
                                                 noise_c_start);
 }
 
