@@ -5,11 +5,21 @@
 
 #include "lib/jxl/render_pipeline/stage_write.h"
 
+#include <jxl/memory_manager.h>
+
+#include <cstdint>
+#include <type_traits>
+
 #include "lib/jxl/alpha.h"
 #include "lib/jxl/base/common.h"
+#include "lib/jxl/base/compiler_specific.h"
+#include "lib/jxl/base/sanitizers.h"
+#include "lib/jxl/base/status.h"
 #include "lib/jxl/dec_cache.h"
+#include "lib/jxl/dec_xyb.h"
+#include "lib/jxl/image.h"
 #include "lib/jxl/image_bundle.h"
-#include "lib/jxl/sanitizers.h"
+#include "lib/jxl/memory_manager_internal.h"
 
 #undef HWY_TARGET_INCLUDE
 #define HWY_TARGET_INCLUDE "lib/jxl/render_pipeline/stage_write.cc"
@@ -21,6 +31,7 @@ namespace jxl {
 namespace HWY_NAMESPACE {
 
 // These templates are not found via ADL.
+using hwy::HWY_NAMESPACE::Add;
 using hwy::HWY_NAMESPACE::Clamp;
 using hwy::HWY_NAMESPACE::Div;
 using hwy::HWY_NAMESPACE::Max;
@@ -30,13 +41,86 @@ using hwy::HWY_NAMESPACE::Or;
 using hwy::HWY_NAMESPACE::Rebind;
 using hwy::HWY_NAMESPACE::ShiftLeftSame;
 using hwy::HWY_NAMESPACE::ShiftRightSame;
+using hwy::HWY_NAMESPACE::VFromD;
+
+// 8x8 ordered dithering pattern from
+// https://en.wikipedia.org/wiki/Ordered_dithering
+// scaled to have an average of 0 and be fully contained in (-0.5, 0.5).
+// Matrix is duplicated in width to avoid inconsistencies or out-of-bound-reads
+// if doing unaligned operations.
+const float kDither[(2 * 8) * 8] = {
+    -0.4921875, 0.0078125, -0.3671875, 0.1328125,  //
+    -0.4609375, 0.0390625, -0.3359375, 0.1640625,  //
+    -0.4921875, 0.0078125, -0.3671875, 0.1328125,  //
+    -0.4609375, 0.0390625, -0.3359375, 0.1640625,  //
+                                                   //
+    0.2578125, -0.2421875, 0.3828125, -0.1171875,  //
+    0.2890625, -0.2109375, 0.4140625, -0.0859375,  //
+    0.2578125, -0.2421875, 0.3828125, -0.1171875,  //
+    0.2890625, -0.2109375, 0.4140625, -0.0859375,  //
+                                                   //
+    -0.3046875, 0.1953125, -0.4296875, 0.0703125,  //
+    -0.2734375, 0.2265625, -0.3984375, 0.1015625,  //
+    -0.3046875, 0.1953125, -0.4296875, 0.0703125,  //
+    -0.2734375, 0.2265625, -0.3984375, 0.1015625,  //
+                                                   //
+    0.4453125, -0.0546875, 0.3203125, -0.1796875,  //
+    0.4765625, -0.0234375, 0.3515625, -0.1484375,  //
+    0.4453125, -0.0546875, 0.3203125, -0.1796875,  //
+    0.4765625, -0.0234375, 0.3515625, -0.1484375,  //
+                                                   //
+    -0.4453125, 0.0546875, -0.3203125, 0.1796875,  //
+    -0.4765625, 0.0234375, -0.3515625, 0.1484375,  //
+    -0.4453125, 0.0546875, -0.3203125, 0.1796875,  //
+    -0.4765625, 0.0234375, -0.3515625, 0.1484375,  //
+                                                   //
+    0.3046875, -0.1953125, 0.4296875, -0.0703125,  //
+    0.2734375, -0.2265625, 0.3984375, -0.1015625,  //
+    0.3046875, -0.1953125, 0.4296875, -0.0703125,  //
+    0.2734375, -0.2265625, 0.3984375, -0.1015625,  //
+                                                   //
+    -0.2578125, 0.2421875, -0.3828125, 0.1171875,  //
+    -0.2890625, 0.2109375, -0.4140625, 0.0859375,  //
+    -0.2578125, 0.2421875, -0.3828125, 0.1171875,  //
+    -0.2890625, 0.2109375, -0.4140625, 0.0859375,  //
+                                                   //
+    0.4921875, -0.0078125, 0.3671875, -0.1328125,  //
+    0.4609375, -0.0390625, 0.3359375, -0.1640625,  //
+    0.4921875, -0.0078125, 0.3671875, -0.1328125,  //
+    0.4609375, -0.0390625, 0.3359375, -0.1640625,  //
+};
+
+using DF = HWY_FULL(float);
+
+// Converts `v` to an appropriate value for the given unsigned type.
+// If the unsigned type is an 8-bit type, performs ordered dithering.
+template <typename T>
+VFromD<Rebind<T, DF>> MakeUnsigned(VFromD<DF> v, size_t x0, size_t y0,
+                                   VFromD<DF> mul) {
+  static_assert(std::is_unsigned<T>::value, "T must be an unsigned type");
+  using DU = Rebind<T, DF>;
+  v = Mul(v, mul);
+  // TODO(veluca): if constexpr with C++17
+  if (sizeof(T) == 1) {
+    size_t pos = (y0 % 8) * (2 * 8) + (x0 % 8);
+#if HWY_TARGET != HWY_SCALAR
+    auto dither = LoadDup128(DF(), kDither + pos);
+#else
+    auto dither = LoadU(DF(), kDither + pos);
+#endif
+    v = Add(v, dither);
+  }
+  v = Clamp(Zero(DF()), v, mul);
+  return DemoteTo(DU(), NearestInt(v));
+}
 
 class WriteToOutputStage : public RenderPipelineStage {
  public:
   WriteToOutputStage(const ImageOutput& main_output, size_t width,
                      size_t height, bool has_alpha, bool unpremul_alpha,
                      size_t alpha_c, Orientation undo_orientation,
-                     const std::vector<ImageOutput>& extra_output)
+                     const std::vector<ImageOutput>& extra_output,
+                     JxlMemoryManager* memory_manager)
       : RenderPipelineStage(RenderPipelineStage::Settings()),
         width_(width),
         height_(height),
@@ -49,7 +133,8 @@ class WriteToOutputStage : public RenderPipelineStage {
         flip_x_(ShouldFlipX(undo_orientation)),
         flip_y_(ShouldFlipY(undo_orientation)),
         transpose_(ShouldTranspose(undo_orientation)),
-        opaque_alpha_(kMaxPixelsPerCall, 1.0f) {
+        opaque_alpha_(kMaxPixelsPerCall, 1.0f),
+        memory_manager_(memory_manager) {
     for (size_t ec = 0; ec < extra_output.size(); ++ec) {
       if (extra_output[ec].callback.IsPresent() || extra_output[ec].buffer) {
         Output extra(extra_output[ec]);
@@ -75,13 +160,13 @@ class WriteToOutputStage : public RenderPipelineStage {
     }
   }
 
-  void ProcessRow(const RowInfo& input_rows, const RowInfo& output_rows,
-                  size_t xextra, size_t xsize, size_t xpos, size_t ypos,
-                  size_t thread_id) const final {
-    JXL_DASSERT(xextra == 0);
-    JXL_DASSERT(main_.run_opaque_ || main_.buffer_);
-    if (ypos >= height_) return;
-    if (xpos >= width_) return;
+  Status ProcessRow(const RowInfo& input_rows, const RowInfo& output_rows,
+                    size_t xextra, size_t xsize, size_t xpos, size_t ypos,
+                    size_t thread_id) const final {
+    JXL_ENSURE(xextra == 0);
+    JXL_ENSURE(main_.run_opaque_ || main_.buffer_);
+    if (ypos >= height_) return true;
+    if (xpos >= width_) return true;
     if (flip_y_) {
       ypos = height_ - 1u - ypos;
     }
@@ -109,6 +194,7 @@ class WriteToOutputStage : public RenderPipelineStage {
         OutputBuffers(extra, thread_id, ypos, xstart, len, line_buffers);
       }
     }
+    return true;
   }
 
   RenderPipelineChannelMode GetChannelMode(size_t c) const final {
@@ -127,7 +213,7 @@ class WriteToOutputStage : public RenderPipelineStage {
 
  private:
   struct Output {
-    Output(const ImageOutput& image_out)
+    explicit Output(const ImageOutput& image_out)
         : pixel_callback_(image_out.callback),
           buffer_(image_out.buffer),
           buffer_size_(image_out.buffer_size),
@@ -166,14 +252,18 @@ class WriteToOutputStage : public RenderPipelineStage {
       JXL_RETURN_IF_ERROR(extra.PrepareForThreads(num_threads));
     }
     temp_out_.resize(num_threads);
-    for (CacheAlignedUniquePtr& temp : temp_out_) {
-      temp = AllocateArray(sizeof(float) * kMaxPixelsPerCall *
-                           main_.num_channels_);
+    for (AlignedMemory& temp : temp_out_) {
+      size_t alloc_size =
+          sizeof(float) * kMaxPixelsPerCall * main_.num_channels_;
+      JXL_ASSIGN_OR_RETURN(temp,
+                           AlignedMemory::Create(memory_manager_, alloc_size));
     }
     if ((has_alpha_ && want_alpha_ && unpremul_alpha_) || flip_x_) {
       temp_in_.resize(num_threads * main_.num_channels_);
-      for (CacheAlignedUniquePtr& temp : temp_in_) {
-        temp = AllocateArray(sizeof(float) * kMaxPixelsPerCall);
+      for (AlignedMemory& temp : temp_in_) {
+        size_t alloc_size = sizeof(float) * kMaxPixelsPerCall;
+        JXL_ASSIGN_OR_RETURN(
+            temp, AlignedMemory::Create(memory_manager_, alloc_size));
       }
     }
     return true;
@@ -204,7 +294,7 @@ class WriteToOutputStage : public RenderPipelineStage {
     float* temp_in[4];
     for (size_t c = 0; c < main_.num_channels_; ++c) {
       size_t tix = thread_id * main_.num_channels_ + c;
-      temp_in[c] = reinterpret_cast<float*>(temp_in_[tix].get());
+      temp_in[c] = temp_in_[tix].address<float>();
       memcpy(temp_in[c], line_buffers[c], sizeof(float) * len);
     }
     auto small_alpha = Set(d, kSmallAlpha);
@@ -227,16 +317,14 @@ class WriteToOutputStage : public RenderPipelineStage {
       FlipX(out, thread_id, len, &xstart, input);
     }
     if (out.data_type_ == JXL_TYPE_UINT8) {
-      uint8_t* JXL_RESTRICT temp =
-          reinterpret_cast<uint8_t*>(temp_out_[thread_id].get());
-      StoreUnsignedRow(out, input, len, temp);
+      uint8_t* JXL_RESTRICT temp = temp_out_[thread_id].address<uint8_t>();
+      StoreUnsignedRow(out, input, len, temp, xstart, ypos);
       WriteToOutput(out, thread_id, ypos, xstart, len, temp);
     } else if (out.data_type_ == JXL_TYPE_UINT16 ||
                out.data_type_ == JXL_TYPE_FLOAT16) {
-      uint16_t* JXL_RESTRICT temp =
-          reinterpret_cast<uint16_t*>(temp_out_[thread_id].get());
+      uint16_t* JXL_RESTRICT temp = temp_out_[thread_id].address<uint16_t>();
       if (out.data_type_ == JXL_TYPE_UINT16) {
-        StoreUnsignedRow(out, input, len, temp);
+        StoreUnsignedRow(out, input, len, temp, xstart, ypos);
       } else {
         StoreFloat16Row(out, input, len, temp);
       }
@@ -251,8 +339,7 @@ class WriteToOutputStage : public RenderPipelineStage {
       }
       WriteToOutput(out, thread_id, ypos, xstart, len, temp);
     } else if (out.data_type_ == JXL_TYPE_FLOAT) {
-      float* JXL_RESTRICT temp =
-          reinterpret_cast<float*>(temp_out_[thread_id].get());
+      float* JXL_RESTRICT temp = temp_out_[thread_id].address<float>();
       StoreFloatRow(out, input, len, temp);
       if (out.swap_endianness_) {
         size_t output_len = len * out.num_channels_;
@@ -269,7 +356,7 @@ class WriteToOutputStage : public RenderPipelineStage {
     float* temp_in[4];
     for (size_t c = 0; c < out.num_channels_; ++c) {
       size_t tix = thread_id * main_.num_channels_ + c;
-      temp_in[c] = reinterpret_cast<float*>(temp_in_[tix].get());
+      temp_in[c] = temp_in_[tix].address<float>();
       if (temp_in[c] != line_buffers[c]) {
         memcpy(temp_in[c], line_buffers[c], sizeof(float) * len);
       }
@@ -289,10 +376,8 @@ class WriteToOutputStage : public RenderPipelineStage {
 
   template <typename T>
   void StoreUnsignedRow(const Output& out, const float* input[4], size_t len,
-                        T* output) const {
+                        T* output, size_t xstart, size_t ypos) const {
     const HWY_FULL(float) d;
-    auto zero = Zero(d);
-    auto one = Set(d, 1.0f);
     auto mul = Set(d, (1u << (out.bits_per_sample_)) - 1);
     const Rebind<T, decltype(d)> du;
     const size_t padding = RoundUpTo(len, Lanes(d)) - len;
@@ -301,43 +386,40 @@ class WriteToOutputStage : public RenderPipelineStage {
     }
     if (out.num_channels_ == 1) {
       for (size_t i = 0; i < len; i += Lanes(d)) {
-        auto v0 = Mul(Clamp(zero, LoadU(d, &input[0][i]), one), mul);
-        StoreU(DemoteTo(du, NearestInt(v0)), du, &output[i]);
+        StoreU(MakeUnsigned<T>(LoadU(d, &input[0][i]), xstart + i, ypos, mul),
+               du, &output[i]);
       }
     } else if (out.num_channels_ == 2) {
       for (size_t i = 0; i < len; i += Lanes(d)) {
-        auto v0 = Mul(Clamp(zero, LoadU(d, &input[0][i]), one), mul);
-        auto v1 = Mul(Clamp(zero, LoadU(d, &input[1][i]), one), mul);
-        StoreInterleaved2(DemoteTo(du, NearestInt(v0)),
-                          DemoteTo(du, NearestInt(v1)), du, &output[2 * i]);
+        StoreInterleaved2(
+            MakeUnsigned<T>(LoadU(d, &input[0][i]), xstart + i, ypos, mul),
+            MakeUnsigned<T>(LoadU(d, &input[1][i]), xstart + i, ypos, mul), du,
+            &output[2 * i]);
       }
     } else if (out.num_channels_ == 3) {
       for (size_t i = 0; i < len; i += Lanes(d)) {
-        auto v0 = Mul(Clamp(zero, LoadU(d, &input[0][i]), one), mul);
-        auto v1 = Mul(Clamp(zero, LoadU(d, &input[1][i]), one), mul);
-        auto v2 = Mul(Clamp(zero, LoadU(d, &input[2][i]), one), mul);
-        StoreInterleaved3(DemoteTo(du, NearestInt(v0)),
-                          DemoteTo(du, NearestInt(v1)),
-                          DemoteTo(du, NearestInt(v2)), du, &output[3 * i]);
+        StoreInterleaved3(
+            MakeUnsigned<T>(LoadU(d, &input[0][i]), xstart + i, ypos, mul),
+            MakeUnsigned<T>(LoadU(d, &input[1][i]), xstart + i, ypos, mul),
+            MakeUnsigned<T>(LoadU(d, &input[2][i]), xstart + i, ypos, mul), du,
+            &output[3 * i]);
       }
     } else if (out.num_channels_ == 4) {
       for (size_t i = 0; i < len; i += Lanes(d)) {
-        auto v0 = Mul(Clamp(zero, LoadU(d, &input[0][i]), one), mul);
-        auto v1 = Mul(Clamp(zero, LoadU(d, &input[1][i]), one), mul);
-        auto v2 = Mul(Clamp(zero, LoadU(d, &input[2][i]), one), mul);
-        auto v3 = Mul(Clamp(zero, LoadU(d, &input[3][i]), one), mul);
-        StoreInterleaved4(DemoteTo(du, NearestInt(v0)),
-                          DemoteTo(du, NearestInt(v1)),
-                          DemoteTo(du, NearestInt(v2)),
-                          DemoteTo(du, NearestInt(v3)), du, &output[4 * i]);
+        StoreInterleaved4(
+            MakeUnsigned<T>(LoadU(d, &input[0][i]), xstart + i, ypos, mul),
+            MakeUnsigned<T>(LoadU(d, &input[1][i]), xstart + i, ypos, mul),
+            MakeUnsigned<T>(LoadU(d, &input[2][i]), xstart + i, ypos, mul),
+            MakeUnsigned<T>(LoadU(d, &input[3][i]), xstart + i, ypos, mul), du,
+            &output[4 * i]);
       }
     }
     msan::PoisonMemory(output + out.num_channels_ * len,
                        sizeof(output[0]) * out.num_channels_ * padding);
   }
 
-  void StoreFloat16Row(const Output& out, const float* input[4], size_t len,
-                       uint16_t* output) const {
+  static void StoreFloat16Row(const Output& out, const float* input[4],
+                              size_t len, uint16_t* output) {
     const HWY_FULL(float) d;
     const Rebind<uint16_t, decltype(d)> du;
     const Rebind<hwy::float16_t, decltype(d)> df16;
@@ -382,8 +464,8 @@ class WriteToOutputStage : public RenderPipelineStage {
                        sizeof(output[0]) * out.num_channels_ * padding);
   }
 
-  void StoreFloatRow(const Output& out, const float* input[4], size_t len,
-                     float* output) const {
+  static void StoreFloatRow(const Output& out, const float* input[4],
+                            size_t len, float* output) {
     const HWY_FULL(float) d;
     if (out.num_channels_ == 1) {
       memcpy(output, input[0], len * sizeof(output[0]));
@@ -454,19 +536,22 @@ class WriteToOutputStage : public RenderPipelineStage {
   bool transpose_;
   std::vector<Output> extra_channels_;
   std::vector<float> opaque_alpha_;
-  std::vector<CacheAlignedUniquePtr> temp_in_;
-  std::vector<CacheAlignedUniquePtr> temp_out_;
+  JxlMemoryManager* memory_manager_;
+  std::vector<AlignedMemory> temp_in_;
+  std::vector<AlignedMemory> temp_out_;
 };
 
+#if JXL_CXX_LANG < JXL_CXX_17
 constexpr size_t WriteToOutputStage::kMaxPixelsPerCall;
+#endif
 
 std::unique_ptr<RenderPipelineStage> GetWriteToOutputStage(
     const ImageOutput& main_output, size_t width, size_t height, bool has_alpha,
     bool unpremul_alpha, size_t alpha_c, Orientation undo_orientation,
-    std::vector<ImageOutput>& extra_output) {
+    std::vector<ImageOutput>& extra_output, JxlMemoryManager* memory_manager) {
   return jxl::make_unique<WriteToOutputStage>(
       main_output, width, height, has_alpha, unpremul_alpha, alpha_c,
-      undo_orientation, extra_output);
+      undo_orientation, extra_output, memory_manager);
 }
 
 // NOLINTNEXTLINE(google-readability-namespace-comments)
@@ -483,47 +568,53 @@ HWY_EXPORT(GetWriteToOutputStage);
 namespace {
 class WriteToImageBundleStage : public RenderPipelineStage {
  public:
-  explicit WriteToImageBundleStage(ImageBundle* image_bundle,
-                                   ColorEncoding color_encoding)
+  explicit WriteToImageBundleStage(
+      ImageBundle* image_bundle, const OutputEncodingInfo& output_encoding_info)
       : RenderPipelineStage(RenderPipelineStage::Settings()),
         image_bundle_(image_bundle),
-        color_encoding_(std::move(color_encoding)) {}
+        color_encoding_(output_encoding_info.color_encoding) {}
 
-  void SetInputSizes(
+  Status SetInputSizes(
       const std::vector<std::pair<size_t, size_t>>& input_sizes) override {
-#if JXL_ENABLE_ASSERT
-    JXL_ASSERT(input_sizes.size() >= 3);
+    JxlMemoryManager* memory_manager = image_bundle_->memory_manager();
+    JXL_ENSURE(input_sizes.size() >= 3);
     for (size_t c = 1; c < input_sizes.size(); c++) {
-      JXL_ASSERT(input_sizes[c].first == input_sizes[0].first);
-      JXL_ASSERT(input_sizes[c].second == input_sizes[0].second);
+      JXL_ENSURE(input_sizes[c].first == input_sizes[0].first);
+      JXL_ENSURE(input_sizes[c].second == input_sizes[0].second);
     }
-#endif
     // TODO(eustas): what should we do in the case of "want only ECs"?
-    image_bundle_->SetFromImage(
-        Image3F(input_sizes[0].first, input_sizes[0].second), color_encoding_);
+    JXL_ASSIGN_OR_RETURN(Image3F tmp,
+                         Image3F::Create(memory_manager, input_sizes[0].first,
+                                         input_sizes[0].second));
+    JXL_RETURN_IF_ERROR(
+        image_bundle_->SetFromImage(std::move(tmp), color_encoding_));
     // TODO(veluca): consider not reallocating ECs if not needed.
     image_bundle_->extra_channels().clear();
     for (size_t c = 3; c < input_sizes.size(); c++) {
-      image_bundle_->extra_channels().emplace_back(input_sizes[c].first,
-                                                   input_sizes[c].second);
+      JXL_ASSIGN_OR_RETURN(ImageF ch,
+                           ImageF::Create(memory_manager, input_sizes[c].first,
+                                          input_sizes[c].second));
+      image_bundle_->extra_channels().emplace_back(std::move(ch));
     }
+    return true;
   }
 
-  void ProcessRow(const RowInfo& input_rows, const RowInfo& output_rows,
-                  size_t xextra, size_t xsize, size_t xpos, size_t ypos,
-                  size_t thread_id) const final {
+  Status ProcessRow(const RowInfo& input_rows, const RowInfo& output_rows,
+                    size_t xextra, size_t xsize, size_t xpos, size_t ypos,
+                    size_t thread_id) const final {
     for (size_t c = 0; c < 3; c++) {
       memcpy(image_bundle_->color()->PlaneRow(c, ypos) + xpos - xextra,
              GetInputRow(input_rows, c, 0) - xextra,
              sizeof(float) * (xsize + 2 * xextra));
     }
     for (size_t ec = 0; ec < image_bundle_->extra_channels().size(); ec++) {
-      JXL_ASSERT(image_bundle_->extra_channels()[ec].xsize() >=
+      JXL_ENSURE(image_bundle_->extra_channels()[ec].xsize() >=
                  xpos + xsize + xextra);
       memcpy(image_bundle_->extra_channels()[ec].Row(ypos) + xpos - xextra,
              GetInputRow(input_rows, 3 + ec, 0) - xextra,
              sizeof(float) * (xsize + 2 * xextra));
     }
+    return true;
   }
 
   RenderPipelineChannelMode GetChannelMode(size_t c) const final {
@@ -539,29 +630,33 @@ class WriteToImageBundleStage : public RenderPipelineStage {
 
 class WriteToImage3FStage : public RenderPipelineStage {
  public:
-  explicit WriteToImage3FStage(Image3F* image)
-      : RenderPipelineStage(RenderPipelineStage::Settings()), image_(image) {}
+  WriteToImage3FStage(JxlMemoryManager* memory_manager, Image3F* image)
+      : RenderPipelineStage(RenderPipelineStage::Settings()),
+        memory_manager_(memory_manager),
+        image_(image) {}
 
-  void SetInputSizes(
+  Status SetInputSizes(
       const std::vector<std::pair<size_t, size_t>>& input_sizes) override {
-#if JXL_ENABLE_ASSERT
-    JXL_ASSERT(input_sizes.size() >= 3);
+    JXL_ENSURE(input_sizes.size() >= 3);
     for (size_t c = 1; c < 3; ++c) {
-      JXL_ASSERT(input_sizes[c].first == input_sizes[0].first);
-      JXL_ASSERT(input_sizes[c].second == input_sizes[0].second);
+      JXL_ENSURE(input_sizes[c].first == input_sizes[0].first);
+      JXL_ENSURE(input_sizes[c].second == input_sizes[0].second);
     }
-#endif
-    *image_ = Image3F(input_sizes[0].first, input_sizes[0].second);
+    JXL_ASSIGN_OR_RETURN(*image_,
+                         Image3F::Create(memory_manager_, input_sizes[0].first,
+                                         input_sizes[0].second));
+    return true;
   }
 
-  void ProcessRow(const RowInfo& input_rows, const RowInfo& output_rows,
-                  size_t xextra, size_t xsize, size_t xpos, size_t ypos,
-                  size_t thread_id) const final {
+  Status ProcessRow(const RowInfo& input_rows, const RowInfo& output_rows,
+                    size_t xextra, size_t xsize, size_t xpos, size_t ypos,
+                    size_t thread_id) const final {
     for (size_t c = 0; c < 3; c++) {
       memcpy(image_->PlaneRow(c, ypos) + xpos - xextra,
              GetInputRow(input_rows, c, 0) - xextra,
              sizeof(float) * (xsize + 2 * xextra));
     }
+    return true;
   }
 
   RenderPipelineChannelMode GetChannelMode(size_t c) const final {
@@ -572,28 +667,30 @@ class WriteToImage3FStage : public RenderPipelineStage {
   const char* GetName() const override { return "WriteI3F"; }
 
  private:
+  JxlMemoryManager* memory_manager_;
   Image3F* image_;
 };
 
 }  // namespace
 
 std::unique_ptr<RenderPipelineStage> GetWriteToImageBundleStage(
-    ImageBundle* image_bundle, ColorEncoding color_encoding) {
+    ImageBundle* image_bundle, const OutputEncodingInfo& output_encoding_info) {
   return jxl::make_unique<WriteToImageBundleStage>(image_bundle,
-                                                   std::move(color_encoding));
+                                                   output_encoding_info);
 }
 
-std::unique_ptr<RenderPipelineStage> GetWriteToImage3FStage(Image3F* image) {
-  return jxl::make_unique<WriteToImage3FStage>(image);
+std::unique_ptr<RenderPipelineStage> GetWriteToImage3FStage(
+    JxlMemoryManager* memory_manager, Image3F* image) {
+  return jxl::make_unique<WriteToImage3FStage>(memory_manager, image);
 }
 
 std::unique_ptr<RenderPipelineStage> GetWriteToOutputStage(
     const ImageOutput& main_output, size_t width, size_t height, bool has_alpha,
     bool unpremul_alpha, size_t alpha_c, Orientation undo_orientation,
-    std::vector<ImageOutput>& extra_output) {
+    std::vector<ImageOutput>& extra_output, JxlMemoryManager* memory_manager) {
   return HWY_DYNAMIC_DISPATCH(GetWriteToOutputStage)(
       main_output, width, height, has_alpha, unpremul_alpha, alpha_c,
-      undo_orientation, extra_output);
+      undo_orientation, extra_output, memory_manager);
 }
 
 }  // namespace jxl
