@@ -13,6 +13,7 @@
 #include <memory>
 #include <thread>  // NOLINT
 
+#include "lib/jxl/base/common.h"
 #include "lib/jxl/base/data_parallel.h"
 
 namespace jpegxl {
@@ -30,13 +31,13 @@ class ThreadPoolInternal {
       size_t num_threads = std::thread::hardware_concurrency()) {
     runner_ =
         JxlThreadParallelRunnerMake(/* memory_manager */ nullptr, num_threads);
-    pool_.reset(new ThreadPool(JxlThreadParallelRunner, runner_.get()));
+    pool_ =
+        jxl::make_unique<ThreadPool>(JxlThreadParallelRunner, runner_.get());
   }
 
   ThreadPoolInternal(const ThreadPoolInternal&) = delete;
   ThreadPoolInternal& operator&(const ThreadPoolInternal&) = delete;
-  // TODO(eustas): avoid unary `&` overload?
-  ThreadPool* operator&() { return pool_.get(); }
+  ThreadPool* get() { return pool_.get(); }
 
  private:
   JxlThreadParallelRunnerPtr runner_;
